@@ -32,60 +32,61 @@
  */
 
 /* ====================================================================
- * Copyright (c) 1996-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
+ *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 /*
@@ -105,7 +106,7 @@
 #include "ap_md5.h"
 #include "ap.h"
 #ifdef CHARSET_EBCDIC
-#include "ebcdic.h"
+#include "ap_ebcdic.h"
 #endif /*CHARSET_EBCDIC*/
 #if HAVE_CRYPT_H
 #include <crypt.h>
@@ -181,7 +182,7 @@ static unsigned char PADDING[64] =
 
 /* MD5 initialization. Begins an MD5 operation, writing a new context.
  */
-API_EXPORT(void) ap_MD5Init(AP_MD5_CTX * context)
+API_EXPORT(void) ap_MD5Init(AP_MD5_CTX *context)
 {
     context->count[0] = context->count[1] = 0;
     /* Load magic initialization constants. */
@@ -195,8 +196,8 @@ API_EXPORT(void) ap_MD5Init(AP_MD5_CTX * context)
    operation, processing another message block, and updating the
    context.
  */
-API_EXPORT(void) ap_MD5Update(AP_MD5_CTX * context, const unsigned char *input,
-			   unsigned int inputLen)
+API_EXPORT(void) ap_MD5Update(AP_MD5_CTX *context, const unsigned char *input,
+			      unsigned int inputLen)
 {
     unsigned int i, idx, partLen;
 
@@ -204,8 +205,10 @@ API_EXPORT(void) ap_MD5Update(AP_MD5_CTX * context, const unsigned char *input,
     idx = (unsigned int) ((context->count[0] >> 3) & 0x3F);
 
     /* Update number of bits */
-    if ((context->count[0] += ((UINT4) inputLen << 3)) < ((UINT4) inputLen << 3))
+    if ((context->count[0] += ((UINT4) inputLen << 3))
+	< ((UINT4) inputLen << 3)) {
 	context->count[1]++;
+    }
     context->count[1] += (UINT4) inputLen >> 29;
 
     partLen = 64 - idx;
@@ -216,41 +219,44 @@ API_EXPORT(void) ap_MD5Update(AP_MD5_CTX * context, const unsigned char *input,
 	memcpy(&context->buffer[idx], input, partLen);
 	MD5Transform(context->state, context->buffer);
 
-	for (i = partLen; i + 63 < inputLen; i += 64)
+	for (i = partLen; i + 63 < inputLen; i += 64) {
 	    MD5Transform(context->state, &input[i]);
+	}
 
 	idx = 0;
     }
-    else
+    else {
 	i = 0;
+    }
 
     /* Buffer remaining input */
     memcpy(&context->buffer[idx], &input[i], inputLen - i);
 #else /*CHARSET_EBCDIC*/
     if (inputLen >= partLen) {
-	ebcdic2ascii_strictly(&context->buffer[idx], input, partLen);
+	ebcdic2ascii(&context->buffer[idx], input, partLen);
 	MD5Transform(context->state, context->buffer);
 
 	for (i = partLen; i + 63 < inputLen; i += 64) {
 	    unsigned char inp_tmp[64];
-	    ebcdic2ascii_strictly(inp_tmp, &input[i], 64);
+	    ebcdic2ascii(inp_tmp, &input[i], 64);
 	    MD5Transform(context->state, inp_tmp);
 	}
 
 	idx = 0;
     }
-    else
+    else {
 	i = 0;
+    }
 
     /* Buffer remaining input */
-    ebcdic2ascii_strictly(&context->buffer[idx], &input[i], inputLen - i);
+    ebcdic2ascii(&context->buffer[idx], &input[i], inputLen - i);
 #endif /*CHARSET_EBCDIC*/
 }
 
 /* MD5 finalization. Ends an MD5 message-digest operation, writing the
    the message digest and zeroizing the context.
  */
-API_EXPORT(void) ap_MD5Final(unsigned char digest[16], AP_MD5_CTX * context)
+API_EXPORT(void) ap_MD5Final(unsigned char digest[16], AP_MD5_CTX *context)
 {
     unsigned char bits[8];
     unsigned int idx, padLen;
@@ -277,10 +283,10 @@ API_EXPORT(void) ap_MD5Final(unsigned char digest[16], AP_MD5_CTX * context)
     /* Pad out to 56 mod 64. */
     idx = (unsigned int) ((context->count[0] >> 3) & 0x3f);
     padLen = (idx < 56) ? (56 - idx) : (120 - idx);
-    ap_MD5Update(context, PADDING, padLen);
+    ap_MD5Update(context, (const unsigned char *)PADDING, padLen);
 
     /* Append length (before padding) */
-    ap_MD5Update(context, bits, 8);
+    ap_MD5Update(context, (const unsigned char *)bits, 8);
 
     /* Store state in digest */
     Encode(digest, context->state, 16);
@@ -406,19 +412,12 @@ static void Decode(UINT4 *output, const unsigned char *input, unsigned int len)
 	    (((UINT4) input[j + 2]) << 16) | (((UINT4) input[j + 3]) << 24);
 }
 
-/*
- * Define the Magic String prefix that identifies a password as being
- * hashed using our algorithm.
- */
-static const char *apr1_id = "$apr1$";
-
 /*
  * The following MD5 password encryption code was largely borrowed from
  * the FreeBSD 3.0 /usr/src/lib/libcrypt/crypt.c file, which is
  * licenced as stated at the top of this file.
  */
-
-static void to64(char *s, unsigned long v, int n)
+API_EXPORT(void) ap_to64(char *s, unsigned long v, int n)
 {
     static unsigned char itoa64[] =         /* 0 ... 63 => ASCII - 64 */
 	"./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
@@ -429,7 +428,8 @@ static void to64(char *s, unsigned long v, int n)
     }
 }
 
-API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
+API_EXPORT(void) ap_MD5Encode(const unsigned char *pw,
+			      const unsigned char *salt,
 			      char *result, size_t nbytes)
 {
     /*
@@ -439,9 +439,12 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
      */
 
     char passwd[120], *p;
-    const char *sp, *ep;
+    const unsigned char *sp, *ep;
     unsigned char final[16];
-    int sl, pl, i;
+    int i;
+    unsigned int sl;
+    int pl;
+    unsigned int pwlen;
     AP_MD5_CTX ctx, ctx1;
     unsigned long l;
 
@@ -455,8 +458,8 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
     /*
      * If it starts with the magic string, then skip that.
      */
-    if (!strncmp(sp, apr1_id, strlen(apr1_id))) {
-	sp += strlen(apr1_id);
+    if (strncmp((char *)sp, AP_MD5PW_ID, AP_MD5PW_IDLEN) == 0) {
+	sp += AP_MD5PW_IDLEN;
     }
 
     /*
@@ -476,15 +479,16 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
      */
     ap_MD5Init(&ctx);
 
+    pwlen = strlen((char *)pw);
     /*
      * The password first, since that is what is most unknown
      */
-    ap_MD5Update(&ctx, pw, strlen(pw));
+    ap_MD5Update(&ctx, pw, pwlen);
 
     /*
      * Then our magic string
      */
-    ap_MD5Update(&ctx, apr1_id, strlen(apr1_id));
+    ap_MD5Update(&ctx, (const unsigned char *) AP_MD5PW_ID, AP_MD5PW_IDLEN);
 
     /*
      * Then the raw salt
@@ -495,12 +499,12 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
      * Then just as many characters of the MD5(pw, salt, pw)
      */
     ap_MD5Init(&ctx1);
-    ap_MD5Update(&ctx1, pw, strlen(pw));
+    ap_MD5Update(&ctx1, pw, pwlen);
     ap_MD5Update(&ctx1, sp, sl);
-    ap_MD5Update(&ctx1, pw, strlen(pw));
+    ap_MD5Update(&ctx1, pw, pwlen);
     ap_MD5Final(final, &ctx1);
-    for(pl = strlen(pw); pl > 0; pl -= 16) {
-	ap_MD5Update(&ctx, final, (pl > 16) ? 16 : pl);
+    for(pl = pwlen; pl > 0; pl -= 16) {
+	ap_MD5Update(&ctx, final, (pl > 16) ? 16 : (unsigned int) pl);
     }
 
     /*
@@ -511,7 +515,7 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
     /*
      * Then something really weird...
      */
-    for (i = strlen(pw); i != 0; i >>= 1) {
+    for (i = pwlen; i != 0; i >>= 1) {
 	if (i & 1) {
 	    ap_MD5Update(&ctx, final, 1);
 	}
@@ -524,9 +528,10 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
      * Now make the output string.  We know our limitations, so we
      * can use the string routines without bounds checking.
      */
-    strcpy(passwd, apr1_id);
-    strncat(passwd, sp, sl);
-    strcat(passwd, "$");
+    ap_cpystrn(passwd, AP_MD5PW_ID, AP_MD5PW_IDLEN + 1);
+    ap_cpystrn(passwd + AP_MD5PW_IDLEN, (char *)sp, sl + 1);
+    passwd[AP_MD5PW_IDLEN + sl]     = '$';
+    passwd[AP_MD5PW_IDLEN + sl + 1] = '\0';
 
     ap_MD5Final(final, &ctx);
 
@@ -538,7 +543,7 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
     for (i = 0; i < 1000; i++) {
 	ap_MD5Init(&ctx1);
 	if (i & 1) {
-	    ap_MD5Update(&ctx1, pw, strlen(pw));
+	    ap_MD5Update(&ctx1, pw, pwlen);
 	}
 	else {
 	    ap_MD5Update(&ctx1, final, 16);
@@ -548,26 +553,26 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
 	}
 
 	if (i % 7) {
-	    ap_MD5Update(&ctx1, pw, strlen(pw));
+	    ap_MD5Update(&ctx1, pw, pwlen);
 	}
 
 	if (i & 1) {
 	    ap_MD5Update(&ctx1, final, 16);
 	}
 	else {
-	    ap_MD5Update(&ctx1, pw, strlen(pw));
+	    ap_MD5Update(&ctx1, pw, pwlen);
 	}
 	ap_MD5Final(final,&ctx1);
     }
 
     p = passwd + strlen(passwd);
 
-    l = (final[ 0]<<16) | (final[ 6]<<8) | final[12]; to64(p, l, 4); p += 4;
-    l = (final[ 1]<<16) | (final[ 7]<<8) | final[13]; to64(p, l, 4); p += 4;
-    l = (final[ 2]<<16) | (final[ 8]<<8) | final[14]; to64(p, l, 4); p += 4;
-    l = (final[ 3]<<16) | (final[ 9]<<8) | final[15]; to64(p, l, 4); p += 4;
-    l = (final[ 4]<<16) | (final[10]<<8) | final[ 5]; to64(p, l, 4); p += 4;
-    l =                    final[11]                ; to64(p, l, 2); p += 2;
+    l = (final[ 0]<<16) | (final[ 6]<<8) | final[12]; ap_to64(p, l, 4); p += 4;
+    l = (final[ 1]<<16) | (final[ 7]<<8) | final[13]; ap_to64(p, l, 4); p += 4;
+    l = (final[ 2]<<16) | (final[ 8]<<8) | final[14]; ap_to64(p, l, 4); p += 4;
+    l = (final[ 3]<<16) | (final[ 9]<<8) | final[15]; ap_to64(p, l, 4); p += 4;
+    l = (final[ 4]<<16) | (final[10]<<8) | final[ 5]; ap_to64(p, l, 4); p += 4;
+    l =                    final[11]                ; ap_to64(p, l, 2); p += 2;
     *p = '\0';
 
     /*
@@ -577,35 +582,3 @@ API_EXPORT(void) ap_MD5Encode(const char *pw, const char *salt,
 
     ap_cpystrn(result, passwd, nbytes - 1);
 }
-
-/*
- * Validate a plaintext password against a smashed one.  Use either
- * crypt() (if available) or ap_MD5Encode(), depending upon the format
- * of the smashed input password.  Return NULL if they match, or
- * an explanatory text string if they don't.
- */
-
-API_EXPORT(char *) ap_validate_password(const char *passwd, const char *hash)
-{
-    char sample[120];
-    char *crypt_pw;
-
-    if (!strncmp(hash, apr1_id, strlen(apr1_id))) {
-	/*
-	 * The hash was created using our custom algorithm.
-	 */
-	ap_MD5Encode(passwd, hash, sample, sizeof(sample));
-    }
-    else {
-	/*
-	 * It's not our algorithm, so feed it to crypt() if possible.
-	 */
-#ifdef WIN32
-	return "crypt() unavailable on Win32, cannot validate password";
-#else
-	crypt_pw = crypt(passwd, hash);
-	ap_cpystrn(sample, crypt_pw, sizeof(sample) - 1);
-#endif
-    }
-    return (strcmp(sample, hash) == 0) ? NULL : "password mismatch";
-}
