@@ -1,58 +1,59 @@
 /* ====================================================================
- * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer. 
+ *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 /*
@@ -113,7 +114,7 @@ module MODULE_VAR_EXPORT includes_module;
 /* XXX: could use ap_table_overlap here */
 static void add_include_vars(request_rec *r, char *timefmt)
 {
-#ifndef WIN32
+#if !defined(WIN32) && !defined(NETWARE)
     struct passwd *pw;
 #endif /* ndef WIN32 */
     table *e = r->subprocess_env;
@@ -454,7 +455,7 @@ static int get_directive(FILE *in, char *dest, size_t len, pool *p)
     }
     /* now get directive */
     while (1) {
-	if (d - dest == len) {
+	if (d == len + dest) {
 	    return 1;
 	}
         *d++ = ap_tolower(c);
@@ -551,7 +552,7 @@ static void parse_string(request_rec *r, const char *in, char *out,
 		    /* zero-length variable name causes just the $ to be copied */
 		    l = 1;
 		}
-		l = (l > end_out - next) ? (end_out - next) : l;
+		l = (l + next > end_out) ? (end_out - next) : l;
 		memcpy(next, expansion, l);
 		next += l;
                 break;
@@ -611,7 +612,7 @@ static int include_cgi(char *s, request_rec *r)
     }
 
     ap_destroy_sub_req(rr);
-#ifndef WIN32
+#if !defined(WIN32) && !defined(NETWARE)
     ap_chdir_file(r->filename);
 #endif
 
@@ -622,12 +623,17 @@ static int include_cgi(char *s, request_rec *r)
  * ensentially ensure that it does not match the regex:
  * (^/|(^|/)\.\.(/|$))
  * XXX: this needs os abstraction... consider c:..\foo in win32
+ * ???: No, c:../foo is not relative to ., it's potentially on another volume
  */
 static int is_only_below(const char *path)
 {
 #ifdef HAVE_DRIVE_LETTERS
     if (path[1] == ':')
 	return 0;
+#endif
+#ifdef NETWARE
+    if (strchr(path, ':'))
+	return 0;
 #endif
     if (path[0] == '/') {
 	return 0;
@@ -712,7 +718,7 @@ static int handle_include(FILE *in, request_rec *r, const char *error, int noexe
                 for (p = r; p != NULL && !founddupe; p = p->main) {
 		    request_rec *q;
 		    for (q = p; q != NULL; q = q->prev) {
-			if ( (strcmp(q->filename, rr->filename) == 0) ||
+			if ( (q->filename && strcmp(q->filename, rr->filename) == 0) ||
 			     (strcmp(q->uri, rr->uri) == 0) ){
 			    founddupe = 1;
 			    break;
@@ -733,7 +739,7 @@ static int handle_include(FILE *in, request_rec *r, const char *error, int noexe
             if (!error_fmt && ap_run_sub_req(rr)) {
                 error_fmt = "unable to include \"%s\" in parsed file %s";
             }
-#ifndef WIN32
+#if !defined(WIN32) && !defined(NETWARE)
             ap_chdir_file(r->filename);
 #endif
             if (error_fmt) {
@@ -889,7 +895,7 @@ static int handle_exec(FILE *in, request_rec *r, const char *error)
                 ap_rputs(error, r);
             }
             /* just in case some stooge changed directories */
-#ifndef WIN32
+#if !defined(WIN32) && !defined(NETWARE)
             ap_chdir_file(r->filename);
 #endif
         }
@@ -901,7 +907,7 @@ static int handle_exec(FILE *in, request_rec *r, const char *error)
                 ap_rputs(error, r);
             }
             /* grumble groan */
-#ifndef WIN32
+#if !defined(WIN32) && !defined(NETWARE)
             ap_chdir_file(r->filename);
 #endif
         }
@@ -1536,6 +1542,7 @@ static int parse_expr(request_rec *r, const char *expr, const char *error)
                     goto RETURN;
                 }
                 break;
+            /* NOTREACHED */
             }
             if (current == (struct parse_node *) NULL) {
                 new->left = root;
@@ -1660,6 +1667,7 @@ static int parse_expr(request_rec *r, const char *expr, const char *error)
                     goto RETURN;
                 }
                 break;
+            /* NOTREACHED */
             }
             if (current == (struct parse_node *) NULL) {
                 new->left = root;
@@ -1951,7 +1959,7 @@ static int handle_if(FILE *in, request_rec *r, const char *error,
     expr = NULL;
     while (1) {
         tag_val = get_tag(r->pool, in, tag, sizeof(tag), 0);
-        if (*tag == '\0') {
+        if (!tag_val || *tag == '\0') {
             return 1;
         }
         else if (!strcmp(tag, "done")) {
@@ -1994,7 +2002,7 @@ static int handle_elif(FILE *in, request_rec *r, const char *error,
     expr = NULL;
     while (1) {
         tag_val = get_tag(r->pool, in, tag, sizeof(tag), 0);
-        if (*tag == '\0') {
+        if (!tag_val || *tag == '\0') {
             return 1;
         }
         else if (!strcmp(tag, "done")) {
@@ -2162,8 +2170,22 @@ static int handle_printenv(FILE *in, request_rec *r, const char *error)
 
 static void send_parsed_content(FILE *f, request_rec *r)
 {
+#ifdef NETWARE
+    /* NetWare has a fixed lengh stack.  Since MAX_STRING_LEN is set
+       to 8k, one call to this function allocates 24k of stack space.
+       During a server-side include evaluation this function is
+       called recusively, allocating 24k each time.  Obviously it 
+       doesn't take long to blow a 64k stack which is the default
+       for Apache for NetWare.  Since MAX_STRING_LEN is used all
+       throughout the Apache code, we should rethink using a default
+       of 8k especially in recursive functions.
+    */
+    char directive[512], error[512];
+    char timefmt[512];
+#else
     char directive[MAX_STRING_LEN], error[MAX_STRING_LEN];
     char timefmt[MAX_STRING_LEN];
+#endif
     int noexec = ap_allow_options(r) & OPT_INCNOEXEC;
     int ret, sizefmt;
     int if_nesting;
@@ -2178,7 +2200,7 @@ static void send_parsed_content(FILE *f, request_rec *r)
     printing = conditional_status = 1;
     if_nesting = 0;
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(NETWARE)
     ap_chdir_file(r->filename);
 #endif
     if (r->args) {              /* add QUERY stuff to env cause it ain't yet */
@@ -2200,6 +2222,7 @@ static void send_parsed_content(FILE *f, request_rec *r)
                 return;
             }
             if (!strcmp(directive, "if")) {
+                ret = 0;
                 if (!printing) {
                     if_nesting++;
                 }
@@ -2208,23 +2231,23 @@ static void send_parsed_content(FILE *f, request_rec *r)
                                     &printing);
                     if_nesting = 0;
                 }
-                continue;
             }
             else if (!strcmp(directive, "else")) {
+                ret = 0;
                 if (!if_nesting) {
                     ret = handle_else(f, r, error, &conditional_status,
                                       &printing);
                 }
-                continue;
             }
             else if (!strcmp(directive, "elif")) {
+                ret = 0;
                 if (!if_nesting) {
                     ret = handle_elif(f, r, error, &conditional_status,
                                       &printing);
                 }
-                continue;
             }
             else if (!strcmp(directive, "endif")) {
+                ret = 0;
                 if (!if_nesting) {
                     ret = handle_endif(f, r, error, &conditional_status,
                                        &printing);
@@ -2232,12 +2255,11 @@ static void send_parsed_content(FILE *f, request_rec *r)
                 else {
                     if_nesting--;
                 }
+            }
+            else if (!printing) {
                 continue;
             }
-            if (!printing) {
-                continue;
-            }
-            if (!strcmp(directive, "exec")) {
+            else if (!strcmp(directive, "exec")) {
                 if (noexec) {
                     ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
 				  "exec used but not allowed in %s",
@@ -2375,7 +2397,7 @@ static int send_parsed_file(request_rec *r)
     }
 
     if ((*state == xbithack_full)
-#if !defined(OS2) && !defined(WIN32)
+#if !defined(OS2) && !defined(WIN32) && !defined(NETWARE)
     /*  OS/2 dosen't support Groups. */
         && (r->finfo.st_mode & S_IXGRP)
 #endif
@@ -2500,7 +2522,7 @@ static int send_shtml_file(request_rec *r)
 
 static int xbithack_handler(request_rec *r)
 {
-#if defined(OS2) || defined(WIN32)
+#if defined(OS2) || defined(WIN32) || defined(NETWARE)
     /* OS/2 dosen't currently support the xbithack. This is being worked on. */
     return DECLINED;
 #else
