@@ -1,5 +1,8 @@
 /* ====================================================================
- * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
+ * The Apache Software License, Version 1.1
+ *
+ * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
+ * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -13,54 +16,55 @@
  *    the documentation and/or other materials provided with the
  *    distribution.
  *
- * 3. All advertising materials mentioning features or use of this
- *    software must display the following acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
+ * 3. The end-user documentation included with the redistribution,
+ *    if any, must include the following acknowledgment:
+ *       "This product includes software developed by the
+ *        Apache Software Foundation (http://www.apache.org/)."
+ *    Alternately, this acknowledgment may appear in the software itself,
+ *    if and wherever such third-party acknowledgments normally appear.
  *
- * 4. The names "Apache Server" and "Apache Group" must not be used to
- *    endorse or promote products derived from this software without
- *    prior written permission. For written permission, please contact
- *    apache@apache.org.
+ * 4. The names "Apache" and "Apache Software Foundation" must
+ *    not be used to endorse or promote products derived from this
+ *    software without prior written permission. For written
+ *    permission, please contact apache@apache.org.
  *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Group.
+ * 5. Products derived from this software may not be called "Apache",
+ *    nor may "Apache" appear in their name, without prior written
+ *    permission of the Apache Software Foundation.
  *
- * 6. Redistributions of any form whatsoever must retain the following
- *    acknowledgment:
- *    "This product includes software developed by the Apache Group
- *    for use in the Apache HTTP server project (http://www.apache.org/)."
- *
- * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
- * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
  * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
+ * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
+ * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  * ====================================================================
  *
  * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Group and was originally based
- * on public domain software written at the National Center for
- * Supercomputing Applications, University of Illinois, Urbana-Champaign.
- * For more information on the Apache Group and the Apache HTTP server
- * project, please see <http://www.apache.org/>.
+ * individuals on behalf of the Apache Software Foundation.  For more
+ * information on the Apache Software Foundation, please see
+ * <http://www.apache.org/>.
  *
+ * Portions of this software are based upon public domain software
+ * originally written at the National Center for Supercomputing Applications,
+ * University of Illinois, Urbana-Champaign.
  */
 
 /* User Tracking Module (Was mod_cookies.c)
+ *
+ * *** IMPORTANT NOTE: This module is not designed to generate 
+ * *** cryptographically secure cookies.  This means you should not 
+ * *** use cookies generated by this module for authentication purposes
  *
  * This Apache module is designed to track users paths through a site.
  * It uses the client-side state ("Cookie") protocol developed by Netscape.
- * It is known to work on Netscape browsers, Microsoft Internet
- * Explorer and others currently being developed.
+ * It is known to work on most browsers.
  *
  * Each time a page is requested we look to see if the browser is sending
  * us a Cookie: header that we previously generated.
@@ -84,17 +88,7 @@
  * can emulate it by adding the following command to your config file
  *       CustomLog filename "%{Cookie}n \"%r\" %t"
  *
- * Notes:
- * 1.  This code now logs the initial transaction (the one that created
- *     the cookie to start with).
- * 2.  This module has been designed to not interfere with other Cookies
- *     your site may be using; just avoid sending out cookies with
- *     the name "Apache=" or things will get confused.
- * 3.  If you want you can modify the Set-Cookie line so that the Cookie
- *     never expires.  You would then get the same Cookie each time the
- *     user revisits your site.
- *
- * Mark Cox, mark@ukweb.com, 6 July 95
+ * Mark Cox, mjc@apache.org, 6 July 95
  *
  * This file replaces mod_cookies.c
  */
@@ -102,7 +96,7 @@
 #include "httpd.h"
 #include "http_config.h"
 #include "http_core.h"
-#if !defined(WIN32) && !defined(MPE)
+#if !defined(WIN32) && !defined(MPE) && !defined(TPF)
 #include <sys/time.h>
 #endif
 
@@ -113,9 +107,18 @@ typedef struct {
     time_t expires;
 } cookie_log_state;
 
+typedef enum {
+    CT_UNSET,
+    CT_NETSCAPE,
+    CT_COOKIE,
+    CT_COOKIE2
+} cookie_type_e;
+
 typedef struct {
     int enabled;
+    cookie_type_e style;
     char *cookie_name;
+    char *cookie_domain;
 } cookie_dir_rec;
 
 /* Define this to allow post-2000 cookies. Cookies use two-digit dates,
@@ -137,7 +140,11 @@ static void make_cookie(request_rec *r)
     struct tms mpe_tms;
 #elif !defined(WIN32)
     struct timeval tv;
+#ifdef NETWARE
+    time_t tz = 0;
+#else
     struct timezone tz = {0, 0};
+#endif /* defined(NETWARE) */
 #endif
     /* 1024 == hardcoded constant */
     char cookiebuf[1024];
@@ -158,6 +165,9 @@ static void make_cookie(request_rec *r)
     ap_snprintf(cookiebuf, sizeof(cookiebuf), "%s.%d%ld%ld", rname,
 		(int) getpid(),
                 (long) r->request_time, (long) mpe_tms.tms_utime);
+#elif defined(NETWARE)
+    ap_snprintf(cookiebuf, sizeof(cookiebuf), "%s.%d%ld%ld", rname,
+        (int) getpid(), (long) r->request_time, (long) clock());                
 #elif defined(WIN32)
     /*
      * We lack gettimeofday() and we lack times(). So we'll use a combination
@@ -179,7 +189,11 @@ static void make_cookie(request_rec *r)
 
     if (cls->expires) {
         struct tm *tms;
-        time_t when = r->request_time + cls->expires;
+        time_t when;
+
+        when = cls->expires;
+        if ((dcfg->style == CT_UNSET) || (dcfg->style == CT_NETSCAPE)) {
+            when += r->request_time;
 
 #ifndef MILLENIAL_COOKIES
         /*
@@ -191,22 +205,43 @@ static void make_cookie(request_rec *r)
         if (when > 946684799)
             when = 946684799;
 #endif
+        }
         tms = gmtime(&when);
 
         /* Cookie with date; as strftime '%a, %d-%h-%y %H:%M:%S GMT' */
-        new_cookie = ap_psprintf(r->pool,
-                "%s=%s; path=/; expires=%s, %.2d-%s-%.2d %.2d:%.2d:%.2d GMT",
-                    dcfg->cookie_name, cookiebuf, ap_day_snames[tms->tm_wday],
-                    tms->tm_mday, ap_month_snames[tms->tm_mon],
-		    tms->tm_year % 100,
-                    tms->tm_hour, tms->tm_min, tms->tm_sec);
+        new_cookie = ap_psprintf(r->pool, "%s=%s; path=/",
+                                 dcfg->cookie_name, cookiebuf);
+        if ((dcfg->style == CT_UNSET) || (dcfg->style == CT_NETSCAPE)) {
+            new_cookie = ap_psprintf(r->pool, "%s; "
+                                     "expires=%s, %.2d-%s-%.2d "
+                                     "%.2d:%.2d:%.2d GMT",
+                                     new_cookie,
+                                     ap_day_snames[tms->tm_wday],
+                                     tms->tm_mday,
+                                     ap_month_snames[tms->tm_mon],
+                                     tms->tm_year % 100,
+                                     tms->tm_hour, tms->tm_min, tms->tm_sec);
+        }
+        else {
+            new_cookie = ap_psprintf(r->pool, "%s; max-age=%d",
+                                     new_cookie, (int) when);
+        }
     }
     else {
 	new_cookie = ap_psprintf(r->pool, "%s=%s; path=/",
 				 dcfg->cookie_name, cookiebuf);
     }
+    if (dcfg->cookie_domain != NULL) {
+        new_cookie = ap_psprintf(r->pool, "%s; domain=%s",
+                                 new_cookie, dcfg->cookie_domain);
+    }
+    if (dcfg->style == CT_COOKIE2) {
+        new_cookie = ap_pstrcat(r->pool, new_cookie, "; version=1", NULL);
+    }
 
-    ap_table_setn(r->headers_out, "Set-Cookie", new_cookie);
+    ap_table_setn(r->headers_out,
+                  (dcfg->style == CT_COOKIE2 ? "Set-Cookie2" : "Set-Cookie"),
+                  new_cookie);
     ap_table_setn(r->notes, "cookie", ap_pstrdup(r->pool, cookiebuf));   /* log first time */
     return;
 }
@@ -222,7 +257,10 @@ static int spot_cookie(request_rec *r)
         return DECLINED;
     }
 
-    if ((cookie = ap_table_get(r->headers_in, "Cookie")))
+    if ((cookie = ap_table_get(r->headers_in,
+                               (dcfg->style == CT_COOKIE2
+                                ? "Cookie2"
+                                : "Cookie"))))
         if ((value = strstr(cookie, dcfg->cookie_name))) {
             char *cookiebuf, *cookieend;
 
@@ -257,6 +295,8 @@ static void *make_cookie_dir(pool *p, char *d)
 
     dcfg = (cookie_dir_rec *) ap_pcalloc(p, sizeof(cookie_dir_rec));
     dcfg->cookie_name = COOKIE_NAME;
+    dcfg->cookie_domain = NULL;
+    dcfg->style = CT_UNSET;
     dcfg->enabled = 0;
     return dcfg;
 }
@@ -269,14 +309,16 @@ static const char *set_cookie_enable(cmd_parms *cmd, void *mconfig, int arg)
     return NULL;
 }
 
-static const char *set_cookie_exp(cmd_parms *parms, void *dummy, const char *arg)
+static const char *set_cookie_exp(cmd_parms *parms, void *dummy,
+                                  const char *arg)
 {
-    cookie_log_state *cls = ap_get_module_config(parms->server->module_config,
-                                              &usertrack_module);
+    cookie_log_state *cls;
     time_t factor, modifier = 0;
     time_t num = 0;
     char *word;
 
+    cls  = ap_get_module_config(parms->server->module_config,
+                                &usertrack_module);
     /* The simple case first - all numbers (we assume) */
     if (ap_isdigit(arg[0]) && ap_isdigit(arg[strlen(arg) - 1])) {
         cls->expires = atol(arg);
@@ -344,13 +386,71 @@ static const char *set_cookie_name(cmd_parms *cmd, void *mconfig, char *name)
     return NULL;
 }
 
+/*
+ * Set the value for the 'Domain=' attribute.
+ */
+static const char *set_cookie_domain(cmd_parms *cmd, void *mconfig, char *name)
+{
+    cookie_dir_rec *dcfg;
+
+    dcfg = (cookie_dir_rec *) mconfig;
+
+    /*
+     * Apply the restrictions on cookie domain attributes.
+     */
+    if (strlen(name) == 0) {
+        return "CookieDomain values may not be null";
+    }
+    if (name[0] != '.') {
+        return "CookieDomain values must begin with a dot";
+    }
+    if (strchr(&name[1], '.') == NULL) {
+        return "CookieDomain values must contain at least one embedded dot";
+    }
+
+    dcfg->cookie_domain = ap_pstrdup(cmd->pool, name);
+    return NULL;
+}
+
+/*
+ * Make a note of the cookie style we should use.
+ */
+static const char *set_cookie_style(cmd_parms *cmd, void *mconfig, char *name)
+{
+    cookie_dir_rec *dcfg;
+
+    dcfg = (cookie_dir_rec *) mconfig;
+
+    if (strcasecmp(name, "Netscape") == 0) {
+        dcfg->style = CT_NETSCAPE;
+    }
+    else if ((strcasecmp(name, "Cookie") == 0)
+             || (strcasecmp(name, "RFC2109") == 0)) {
+        dcfg->style = CT_COOKIE;
+    }
+    else if ((strcasecmp(name, "Cookie2") == 0)
+             || (strcasecmp(name, "RFC2965") == 0)) {
+        dcfg->style = CT_COOKIE2;
+    }
+    else {
+        return ap_psprintf(cmd->pool, "Invalid %s keyword: '%s'",
+                           cmd->cmd->name, name);
+    }
+
+    return NULL;
+}
+
 static const command_rec cookie_log_cmds[] = {
-    {"CookieExpires", set_cookie_exp, NULL, RSRC_CONF, TAKE1,
+    {"CookieExpires", set_cookie_exp, NULL, OR_FILEINFO, TAKE1,
      "an expiry date code"},
     {"CookieTracking", set_cookie_enable, NULL, OR_FILEINFO, FLAG,
      "whether or not to enable cookies"},
     {"CookieName", set_cookie_name, NULL, OR_FILEINFO, TAKE1,
      "name of the tracking cookie"},
+    {"CookieDomain", set_cookie_domain, NULL, OR_FILEINFO, TAKE1,
+     "domain to which this cookie applies"},
+    {"CookieStyle", set_cookie_style, NULL, OR_FILEINFO, TAKE1,
+     "'Netscape', 'Cookie' (RFC2109), or 'Cookie2' (RFC2965)"},
     {NULL}
 };
 
@@ -375,3 +475,6 @@ module MODULE_VAR_EXPORT usertrack_module = {
     NULL,                       /* child_exit */
     NULL                        /* post read-request */
 };
+
+
+
