@@ -4,6 +4,7 @@
 #define PLATFORM "OS/2"
 #define HAVE_CANONICAL_FILENAME
 #define HAVE_DRIVE_LETTERS
+#define HAVE_UNC_PATHS
 
 /*
  * This file in included in all Apache source code. It contains definitions
@@ -40,6 +41,14 @@ char *ap_os_error_message(int err);
 
 /* OS/2 doesn't have symlinks so S_ISLNK is always false */
 #define S_ISLNK(m) 0
+#define lstat(x, y) stat(x, y)
+
+#define isinf(n) (!isfinite(n))
+#define HAVE_ISINF
+#define HAVE_ISNAN
+
+/* strtol() correctly returns ERANGE on overflow, use it */
+#define ap_strtol strtol
 
 /* Dynamic loading functions */
 #define     ap_os_dso_handle_t  unsigned long
