@@ -1,7 +1,7 @@
 /* ====================================================================
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 2000 The Apache Software Foundation.  All rights
+ * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -75,6 +75,7 @@ static TPF_FD_LIST *tpf_fds = NULL;
 
 void *tpf_shm_static_ptr = NULL;
 unsigned short zinet_model;
+char *argv_ptr = NULL;
 
 static FILE *sock_fp;
 
@@ -103,6 +104,7 @@ int tpf_select(int maxfds, fd_set *reads, fd_set *writes, fd_set *excepts, struc
     ap_check_signals();
     if ((no_reads + no_writes + no_excepts == 0) &&
         (tv) && (tv->tv_sec + tv->tv_usec != 0)) {
+#ifdef TPF_HAVE_SAWNC
         /* TPF's select immediately returns if the sum of
            no_reads, no_writes, and no_excepts is zero.
            This means that the select calls in http_main.c
@@ -110,7 +112,6 @@ int tpf_select(int maxfds, fd_set *reads, fd_set *writes, fd_set *excepts, struc
            The following code makes TPF's select work a little closer
            to everyone else's select:
         */
-#ifndef NO_SAWNC
         struct ev0bk evnblock;
 
         timeout = tv->tv_sec;
@@ -202,6 +203,17 @@ int ap_tpf_spawn_child(pool *p, int (*func) (void *, child_info *),
    TPF_FORK_CHILD           *cld = (TPF_FORK_CHILD *) data;
    array_header             *env_arr = ap_table_elts ((array_header *) cld->subprocess_env);
    table_entry              *elts = (table_entry *) env_arr->elts;
+#ifdef TPF_FORK_EXTENDED
+#define WHITE " \t\n"
+#define MAXARGC 49
+   char                     *arguments;
+   char                     *args[MAXARGC + 1];
+   char                     **envp = NULL;
+   pool                     *subpool = NULL;
+
+#include "util_script.h"
+
+#endif /* TPF_FORK_EXTENDED */ 
 
    if (func) {
       if (result=func(data, NULL)) {
@@ -233,12 +245,22 @@ int ap_tpf_spawn_child(pool *p, int (*func) (void *, child_info *),
       dup2(err_fds[1], STDERR_FILENO);
    }
 
+/* set up environment variables for the tpf_fork */
    if (cld->subprocess_env) {
+#ifdef TPF_FORK_EXTENDED
+   /* with extended tpf_fork( ) we pass the pointer to a list of pointers */
+   /* that point to "key=value" strings for each env variable             */ 
+      subpool = ap_make_sub_pool(p);
+      envp = ap_create_environment(subpool, cld->subprocess_env);
+#else
+   /* without extended tpf_fork( ) we setenv( ) each env variable */
+   /* so the child inherits them                                  */
       for (i = 0; i < env_arr->nelts; ++i) {
            if (!elts[i].key)
                continue;
            setenv (elts[i].key, elts[i].val, 1);
        }
+#endif /* TPF_FORK_EXTENDED */
    }
 
    fork_input.program = (const char*) cld->filename;
@@ -248,8 +270,22 @@ int ap_tpf_spawn_child(pool *p, int (*func) (void *, child_info *),
    fork_input.ebw_data = NULL;
    fork_input.parm_data = NULL;
 
+#ifdef TPF_FORK_EXTENDED
+   /* use a copy of cld->filename because strtok is destructive */
+   arguments = ap_pstrdup(p, cld->filename);
+   args[0] = strtok(arguments, WHITE);
 
+   for (i = 0; i < MAXARGC && args[i] ; i++) {
+       args[i + 1] = strtok(NULL, WHITE);
+   }
+   args[MAXARGC] = NULL;
+
+   if ((pid = tpf_fork(&fork_input,
+                       (const char **)args,
+                       (const char **)envp)) < 0) {
+#else
    if ((pid = tpf_fork(&fork_input)) < 0) {
+#endif /* TPF_FORK_EXTENDED */
        save_errno = errno;
        if (pipe_out) {
            close(out_fds[0]);
@@ -264,6 +300,11 @@ int ap_tpf_spawn_child(pool *p, int (*func) (void *, child_info *),
        pid = 0;
    }
 
+#ifdef TPF_FORK_EXTENDED
+   if (subpool) {
+       ap_destroy_pool(subpool);
+   }
+#else 
    if (cld->subprocess_env) {
        for (i = 0; i < env_arr->nelts; ++i) {
             if (!elts[i].key)
@@ -271,6 +312,7 @@ int ap_tpf_spawn_child(pool *p, int (*func) (void *, child_info *),
             unsetenv (elts[i].key);
        }
    }
+#endif /* TPF_FORK_EXTENDED */
 
    if (pipe_out) {
        close(out_fds[1]);
@@ -353,8 +395,12 @@ pid_t os_fork(server_rec *s, int slot)
     fork_input.prog_type = TPF_FORK_NAME;
     fork_input.istream = TPF_FORK_IS_BALANCE;
     fork_input.ebw_data_length = sizeof(input_parms);
-    fork_input.parm_data = "-x";
+    fork_input.parm_data = argv_ptr;
+#ifdef TPF_FORK_EXTENDED
+    return tpf_fork(&fork_input, NULL, NULL);
+#else
     return tpf_fork(&fork_input);
+#endif /* TPF_FORK_EXTENDED */
 }
 
 void ap_tpf_zinet_checks(int standalone,
@@ -402,25 +448,25 @@ void ap_tpf_zinet_checks(int standalone,
 int os_check_server(char *server) {
     int *current_acn;
 
-    if (zinet_model == INETD_IDCF_MODEL_NOLISTEN) {
-        /* if NOLISTEN model, check with ZINET for status */
-        if (inetd_getServerStatus(server) == INETD_SERVER_STATUS_INACTIVE) {
-            return 1;
-        }
-        /* and check that program activation number hasn't changed */
+    /* check that the program activation number hasn't changed */
         current_acn = (int *)cinfc_fast(CINFC_CMMACNUM);
         if (ecbp2()->ce2acn != *current_acn) {
-            return 1;
+        return 1;  /* shutdown */
         }
 
-    } else {
-        /* if DAEMON model, just make sure parent is still around */
+    /* check our InetD status */
+    if (inetd_getServerStatus(server) != INETD_SERVER_STATUS_ACTIVE) {
+        return 1;  /* shutdown */
+    }
+
+    /* if DAEMON model, make sure parent is still around */
+    if (zinet_model == INETD_IDCF_MODEL_DAEMON) {
         if (getppid() == 1) {
-            return 1;
+            return 1;  /* shutdown */
         }
     }
 
-    return 0;
+    return 0;  /* keep on running... */
 }
 
 void os_note_additional_cleanups(pool *p, int sd) {
@@ -434,6 +480,23 @@ void os_note_additional_cleanups(pool *p, int sd) {
     fcntl(sd,F_SETFD,FD_CLOEXEC);
 }
 
+void ap_tpf_save_argv(int argc, char **argv) {
+
+    int i, len = 3;      /* 3 for "-x "   */
+
+    for (i = 1; i < argc; i++) {    /* find len for calloc */
+         len += strlen (argv[i]);
+         ++len;                     /* 1 for blank */
+    }
+
+    argv_ptr = malloc(len + 1);
+    strcpy(argv_ptr, "-x");
+    for (i = 1; i < argc; i++) {
+         strcat(argv_ptr, " ");
+         strcat(argv_ptr, argv[i]);
+    }
+}
+
 void os_tpf_child(APACHE_TPF_INPUT *input_parms) {
     tpf_child = 1;
     ap_my_generation = input_parms->generation;
@@ -714,3 +777,45 @@ int killpg(pid_t pgrp, int sig)
 
     return(0);
 }
+
+/*
+   This function augments http_main's show_compile_settings function.
+   This way definitions that are only shown on TPF won't clutter up
+   main line code.
+*/
+void show_os_specific_compile_settings(void)
+{
+
+#ifdef USE_TPF_SCOREBOARD
+    printf(" -D USE_TPF_SCOREBOARD\n");
+#endif
+
+#ifdef TPF_FORK_EXTENDED
+    printf(" -D TPF_FORK_EXTENDED\n"); 
+#endif
+
+#ifdef TPF_HAVE_NONSOCKET_SELECT
+    printf(" -D TPF_HAVE_NONSOCKET_SELECT\n"); 
+#endif
+
+#ifdef TPF_NO_NONSOCKET_SELECT 
+    printf(" -D TPF_NO_NONSOCKET_SELECT\n"); 
+#endif
+
+#ifdef TPF_HAVE_SAWNC
+    printf(" -D TPF_HAVE_SAWNC\n"); 
+#endif
+
+#ifdef TPF_NO_SAWNC
+    printf(" -D TPF_NO_SAWNC\n"); 
+#endif
+ 
+#ifdef TPF_HAVE_NSD
+    printf(" -D TPF_HAVE_NSD\n");
+#endif
+ 
+#ifdef HAVE_SYSLOG
+    printf(" -D HAVE_SYSLOG\n");
+#endif
+
+}
